@@ -28,6 +28,11 @@ interface (currently minus seek() and tell()) on top of primitive
 _read() and _write() methods.  Subclasses may implement either or
 both of these methods to obtain all the higher-level file behaviors.
 
+Two methods are provided when code expects to deal with file-like objects:
+    
+    * is_filelike(obj):   checks that an object is file-like
+    * to_filelike(obj):   wraps a variety of objects in a file-like interface
+
 On top of this framework are built a collection of useful classes,
 including:
     
@@ -39,7 +44,7 @@ including:
                            
     * DecryptFile:    on-the-fly reading and writing to an encrypted file
                       (using PEP272 cipher API)
-                     
+
  
 As an example of the type of thing this module is designed to achieve, here's
 an example of using the DecryptFile class to transparently access an encrypted
@@ -53,12 +58,12 @@ file:
     
 The object in <f> now behaves as a file-like object, transparently decrypting
 the file on-the-fly as it is read.
-    
+
 """ 
 
 __ver_major__ = 0
 __ver_minor__ = 1
-__ver_patch__ = 0
+__ver_patch__ = 1
 __ver_sub__ = ""
 __version__ = "%d.%d.%d%s" % (__ver_major__,__ver_minor__,
                               __ver_patch__,__ver_sub__)
@@ -66,6 +71,10 @@ __version__ = "%d.%d.%d%s" % (__ver_major__,__ver_minor__,
 
 
 import unittest
+import StringIO
+
+# TODO:  automatically handle encoding/decoding of unicode strings?
+# TODO:  is_filelike() and to_filelike() functions
 
 
 class FileLikeBase:
@@ -270,6 +279,97 @@ class FileLikeBase:
 
 
 
+def is_filelike(obj,mode="rw"):
+    """Test whether an object implements the file-like interface.
+    
+    <obj> must be the object to be tested, and <mode> a file access
+    mode such as "r", "w" or "rw".  This function returns True if 
+    the given object implements the full reading/writing interface
+    as required by the given mode, and False otherwise.
+    
+    If <mode> is not specified, it deaults to "rw" - that is,
+    checking that the full file interface is supported.
+    
+    This method is not intended for checking basic functionality such as
+    existance of read(), but for ensuring the richer interface is
+    available.  If only read() or write() is needed, it's probably
+    simpler to (a) catch the AttributeError, or (b) use to_filelike(obj)
+    to ensure a suitable object.
+    """
+    if mode == "" or not isinstance(mode,basestring):
+        raise ValueError("Invalid mode spec: %s" % (mode,))
+    for c in mode:
+        if c not in ("r","w","a","+"):
+            raise ValueError("Invalid mode spec: %s" % (mode,))    
+    # Check reading interface
+    if "r" in mode:
+        # Special-case for FileLikeBase subclasses
+        if isinstance(obj,FileLikeBase):
+            if not hasattr(obj,"_read"):
+                return False
+            if obj._read.im_class is FileLikeBase:
+                return False
+        else:
+            attrs = ("read","readline","readlines","__iter__",)
+            for a in attrs:
+                if not hasattr(obj,a):
+                    return False
+    # Check writing interface
+    if "w" in mode or "a" in mode:
+        # Special-case for FileLikeBase subclasses
+        if isinstance(obj,FileLikeBase):
+            if not hasattr(obj,"_write"):
+                return False
+            if obj._write.im_class is FileLikeBase:
+                return False
+        else:
+            attrs = ("write","writelines","close")
+            for a in attrs:
+                if not hasattr(obj,a):
+                    return False
+    return True
+
+
+def to_filelike(obj,mode="rw"):
+    """Convert <obj> to a file-like object if possible.
+    
+    This method takes an arbitrary object <obj>, and attempts to
+    wrap it in a file-like interface.  This will results in the
+    object itself if it is already file-like, or some sort of
+    wrapper class otherwise.
+    
+    <mode>, if provided, should specify how the results object
+    will be accessed - "r" for read, "w" for write, or "rw" for
+    both.
+    
+    If the object cannot be converted, ValueError is raised.
+    """
+    # File-like objects are sutiable on their own
+    if is_filelike(obj,mode):
+        return obj
+    # Strings can be wrapped using StringIO
+    if isinstance(obj,basestring):
+        return StringIO.StringIO(obj)
+    # Anything with read() and/or write() can be trivially wrapped
+    hasRead = hasattr(obj,"read")
+    hasWrite = hasattr(obj,"write")
+    if "r" in mode:
+        if "w" in mode or "a" in mode:
+            if hasRead and hasWrite:
+                return FileWrapper(obj)
+        else:
+            if hasRead:
+                return FileWrapper(obj)
+    if "w" in mode or "a" in mode:
+        if hasWrite:
+            return FileWrapper(obj)
+    # TODO: lots more could be done here...
+    raise ValueError("Could not make object file-like: %s", (obj,))
+
+## TODO: unittests for is_filelike and to_filelike
+
+##  TODO:  maybe move these into a sub-package, filelike.wrappers ?
+
 class FileWrapper(FileLikeBase):
     """Base class for objects that wrap a file-like object.
     
@@ -691,4 +791,4 @@ def testsuite():
 
 # Run regression tests when called from comand-line
 if __name__ == "__main__":
-    UnitTest.TextTestRunner().run(testsuite())
\ No newline at end of file
+    UnitTest.TextTestRunner().run(testsuite())
